import { TextDocument } from 'vscode-languageserver-textdocument';
import { FormattingOptions, TextEdit } from 'vscode-languageserver-types';
import { CustomFormatterOptions, LanguageSettings } from '../yamlLanguageService';
export declare class YAMLFormatter {
    private formatterEnabled;
    private yamlVersion;
    private customTags;
    configure(shouldFormat: LanguageSettings): void;
    format(document: TextDocument, options?: Partial<FormattingOptions> & CustomFormatterOptions): Promise<TextEdit[]>;
}
